package speiger.src.coreengine.rendering.gui;

import java.io.BufferedReader;
import java.util.HashMap;
import java.util.Map;

import speiger.src.collections.chars.maps.impl.hash.Char2ObjectOpenHashMap;
import speiger.src.collections.chars.maps.interfaces.Char2ObjectMap;
import speiger.src.coreengine.assets.AssetLocation;
import speiger.src.coreengine.assets.IAsset;
import speiger.src.coreengine.rendering.gui.renderer.FontRenderer;
import speiger.src.coreengine.rendering.gui.renderer.IFontRenderer.CharInstance;
import speiger.src.coreengine.rendering.textures.SimpleTexture;
import speiger.src.coreengine.rendering.textures.TextureManager;

public class FontLoader
{	
	public static FontRenderer createFont(AssetLocation location, String name)
	{
		return createFont(location, name, 0.35F);
	}
	
	public static FontRenderer createFont(AssetLocation location, String name, float scale)
	{
		try(IAsset asset = TextureManager.INSTANCE.getManager().getAsset(location.subAsset(name+".fnt")))
		{
			Char2ObjectMap<CharInstance>[] maps = new Char2ObjectMap[]{new Char2ObjectOpenHashMap<CharInstance>(), new Char2ObjectOpenHashMap<CharInstance>()};
			BufferedReader reader = asset.getStringReader();
			FontInfo info = new FontInfo(convert(reader.readLine().split(", ")));
			String value = null;
			while((value = getNextValidLine(reader)) != null)
			{
				Map<String, Integer> dataMap = convert(value.split(", "));
				CharInstance instance = createChar(dataMap, info);
				instance.scale(scale);
				maps[instance.isBold() ? 1 : 0].putIfAbsent(instance.getCharacter(), instance);
			}
			info.scale(scale);
			return new FontRenderer(maps, new SimpleTexture(location.subAsset(name+"-Texture.png")), info.fontHeight, info.lineHeight);
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return null;
	}
	
	static String getNextValidLine(BufferedReader reader) throws Exception
	{
		String line = reader.readLine();
		if(line == null)
		{
			return null;
		}
		else if(line.isEmpty() || line.startsWith("//"))
		{
			return getNextValidLine(reader);
		}
		return line;
	}
	
	static CharInstance createChar(Map<String, Integer> data, FontInfo info)
	{
		char character = (char)data.get("letter").intValue();
		int minX = data.get("minX");
		int minY = data.get("minY");
		int maxX = data.get("maxX");
		int maxY = data.get("maxY");
		return new CharInstance(character, maxX - minX, maxY - minY, info.getTextureU(minX), info.getTextureV(minY), info.getTextureU(maxX), info.getTextureV(maxY), maxX - minX, data.getOrDefault("bold", 0).intValue() == 1);
	}
	
	static Map<String, Integer> convert(String[] data)
	{
		Map<String, Integer> map = new HashMap<String, Integer>();
		for(String s : data)
		{
			int index = s.indexOf("=");
			if(index != -1)
			{
				String[] split = s.split("=");
				map.put(split[0], Integer.parseInt(split[1]));
			}
		}
		return map;
	}
	
	public static class FontInfo
	{
		int height;
		int width;
		float fontHeight;
		float lineHeight;
		
		public FontInfo(Map<String, Integer> data)
		{
			width = data.get("textureWidth");
			height = data.get("textureHeight");
			fontHeight = data.get("fontHeight");
			lineHeight = data.get("base");
		}
		
		public void scale(float scale)
		{
			fontHeight *= scale;
			lineHeight *= scale;
		}
		
		public float getTextureU(float value)
		{
			return value / width;
		}
		
		public float getTextureV(float value)
		{
			return value / height;
		}
	}
}
